"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var lodash_clonedeep_1 = tslib_1.__importDefault(require("lodash.clonedeep"));
var method_1 = require("../utils/method");
var OrphanHttpServiceHandle_1 = require("./OrphanHttpServiceHandle");
var createReduxStore_1 = require("../utils/createReduxStore");
var RequestAction_1 = require("../../libs/RequestAction");
var instanceName_1 = require("../utils/instanceName");
var dev_1 = require("../utils/dev");
var BaseHttpService = /** @class */ (function () {
    function BaseHttpService(config) {
        this.caches = {};
        this.config = config;
    }
    BaseHttpService.prototype.action = function (fn) {
        var instanceName = instanceName_1.getInstanceName();
        if (!dev_1.useProxy()) {
            instanceName += '_' + instanceName_1.increaseActionCounter();
        }
        return new RequestAction_1.RequestAction(fn, instanceName, this.withCache.bind(this));
    };
    BaseHttpService.prototype.getAsync = function (config) {
        var service = new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, method_1.METHOD.get, this);
        return this.withCache(service.collect());
    };
    BaseHttpService.prototype.postAsync = function (config) {
        var service = new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, method_1.METHOD.post, this);
        return this.withCache(service.collect());
    };
    BaseHttpService.prototype.putAsync = function (config) {
        var service = new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, method_1.METHOD.put, this);
        return this.withCache(service.collect());
    };
    BaseHttpService.prototype.deleteAsync = function (config) {
        var service = new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, method_1.METHOD.delete, this);
        return this.withCache(service.collect());
    };
    BaseHttpService.prototype.generateCacheKey = function (action) {
        return JSON.stringify([
            action.reducerName,
            action.type,
            action.uri,
            action.method,
            action.body,
            action.query,
            action.requestOptions,
        ]);
    };
    BaseHttpService.prototype.withCache = function (action) {
        var _this = this;
        if (action.useCache) {
            // The cacheKey only will be generated when useCache is set to true to improve performance
            action.cacheKey = this.generateCacheKey(action);
            var item_1 = this.caches[action.cacheKey];
            if (item_1 && Date.now() <= item_1.timestamp) {
                var promise = new Promise(function (resolve) {
                    var fakeAction = tslib_1.__assign(tslib_1.__assign({}, action), { type: action.type.success, response: lodash_clonedeep_1.default(item_1.response), effect: action.onSuccess });
                    _this.next(fakeAction);
                    _this.triggerShowSuccess(fakeAction, action.successText);
                    resolve(fakeAction);
                });
                var wrapPromise = promise;
                wrapPromise.cancel = function () { };
                return wrapPromise;
            }
        }
        // In case user toggle cache flag
        // In case data is expired
        Reflect.deleteProperty(this.caches, action.cacheKey);
        return this.runAction(action);
    };
    BaseHttpService.prototype.collectResponse = function (action) {
        if (action.useCache && action.cacheMillSeconds > 0) {
            this.caches[action.cacheKey] = {
                timestamp: Date.now() + action.cacheMillSeconds,
                response: lodash_clonedeep_1.default(action.response),
            };
        }
    };
    BaseHttpService.prototype.next = function (action) {
        if (action.type) {
            createReduxStore_1.getStore().dispatch(action);
        }
    };
    BaseHttpService.prototype.triggerShowSuccess = function (okResponse, successText) {
        if (successText) {
            this.config.onShowSuccess(successText, okResponse);
        }
    };
    BaseHttpService.prototype.triggerShowError = function (errorResponse, hideError) {
        if (!errorResponse.message) {
            return;
        }
        var showError;
        if (typeof hideError === 'boolean') {
            showError = !hideError;
        }
        else {
            showError = !hideError(errorResponse);
        }
        if (showError) {
            this.config.onShowError(errorResponse.message, errorResponse);
        }
    };
    return BaseHttpService;
}());
exports.BaseHttpService = BaseHttpService;
