"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var HttpServiceBuilder = /** @class */ (function () {
    function HttpServiceBuilder(config) {
        this.types = {
            prepare: '',
            success: '',
            fail: '',
        };
        this.config = config;
    }
    HttpServiceBuilder.prototype.query = function (query) {
        this.config.query = query;
        return this;
    };
    HttpServiceBuilder.prototype.body = function (body) {
        this.config.body = body;
        return this;
    };
    HttpServiceBuilder.prototype.successText = function (text) {
        this.config.successText = text;
        return this;
    };
    HttpServiceBuilder.prototype.failText = function (text) {
        this.config.failText = text;
        return this;
    };
    HttpServiceBuilder.prototype.requestOptions = function (options) {
        this.config.requestOptions = options;
        return this;
    };
    HttpServiceBuilder.prototype.hideError = function (is) {
        this.config.hideError = is;
        return this;
    };
    HttpServiceBuilder.prototype.cache = function (millSeconds, useCache) {
        if (useCache === void 0) { useCache = true; }
        this.config.useCache = useCache;
        this.config.cacheMillSeconds = millSeconds;
        return this;
    };
    HttpServiceBuilder.prototype.payload = function (payload) {
        this.config.payload = payload;
        // @ts-ignore
        return this;
    };
    HttpServiceBuilder.prototype.withMeta = function (param) {
        this.config.metaKey = param;
        return this;
    };
    ;
    HttpServiceBuilder.prototype.onPrepare = function (fn) {
        this.config.onPrepare = fn;
        return this;
    };
    HttpServiceBuilder.prototype.onSuccess = function (fn) {
        this.config.onSuccess = fn;
        return this;
    };
    HttpServiceBuilder.prototype.onFail = function (fn) {
        this.config.onFail = fn;
        return this;
    };
    HttpServiceBuilder.prototype.collect = function (types) {
        var config = this.config;
        var action = {
            uri: config.uri,
            type: types,
            method: config.method,
            reducerName: config.instanceName,
            payload: config.payload === undefined ? {} : config.payload,
            body: config.body || {},
            query: config.query || {},
            successText: config.successText || '',
            failText: config.failText || '',
            hideError: config.hideError || false,
            requestOptions: config.requestOptions || {},
            metaKey: config.metaKey === undefined ? true : config.metaKey,
            onPrepare: config.onPrepare || null,
            onSuccess: config.onSuccess || null,
            onFail: config.onFail || null,
            useCache: config.useCache || false,
            cacheMillSeconds: config.cacheMillSeconds || 0,
            cacheKey: '',
        };
        return action;
    };
    return HttpServiceBuilder;
}());
exports.HttpServiceBuilder = HttpServiceBuilder;
