"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var axios_1 = tslib_1.__importDefault(require("axios"));
var BaseHttpService_1 = require("../core/service/BaseHttpService");
var method_1 = require("../core/utils/method");
var OrphanHttpServiceHandle_1 = require("../core/service/OrphanHttpServiceHandle");
var HttpService = /** @class */ (function (_super) {
    tslib_1.__extends(HttpService, _super);
    function HttpService(config) {
        var _this = _super.call(this, config) || this;
        _this.config = config;
        _this.httpHandle = axios_1.default.create(tslib_1.__assign({ baseURL: config.baseUrl, timeout: 20000, withCredentials: false, responseType: 'json' }, config.requestConfig));
        return _this;
    }
    HttpService.prototype.patchAsync = function (config) {
        var service = new OrphanHttpServiceHandle_1.OrphanHttpServiceHandle(config, method_1.METHOD.patch, this);
        return this.withCache(service.collect());
    };
    HttpService.prototype.clone = function (config) {
        return new HttpService(tslib_1.__assign(tslib_1.__assign({}, this.config), config));
    };
    HttpService.prototype.runAction = function (action) {
        var _this = this;
        var _a, _b;
        (_b = (_a = this.config).beforeSend) === null || _b === void 0 ? void 0 : _b.call(_a, action);
        var _c = action.type, prepare = _c.prepare, success = _c.success, fail = _c.fail;
        var source = axios_1.default.CancelToken.source();
        var requestOptions = tslib_1.__assign(tslib_1.__assign({ url: action.uri, params: action.query, cancelToken: source.token, method: action.method }, action.requestOptions), { headers: tslib_1.__assign(tslib_1.__assign({}, this.config.headers(action)), action.requestOptions.headers) });
        var successInvoked = false;
        if ([method_1.METHOD.post, method_1.METHOD.put, method_1.METHOD.delete, method_1.METHOD.patch].includes(action.method)) {
            requestOptions.data = action.body;
        }
        this.next(tslib_1.__assign(tslib_1.__assign({}, action), { type: prepare, effect: action.onPrepare }));
        var promise = this.httpHandle.request(requestOptions)
            .then(function (response) {
            if (_this.config.isSuccess && !_this.config.isSuccess(response)) {
                return Promise.reject({
                    response: response,
                });
            }
            if (_this.config.transformSuccessData) {
                response.data = _this.config.transformSuccessData(response.data, response.headers);
            }
            var okResponse = tslib_1.__assign(tslib_1.__assign({}, action), { type: success, response: response.data, effect: action.onSuccess });
            successInvoked = true;
            _this.collectResponse(okResponse);
            _this.next(okResponse);
            _this.triggerShowSuccess(okResponse, action.successText);
            return Promise.resolve(okResponse);
        })
            .catch(function (error) {
            if (successInvoked) {
                return Promise.reject(error);
            }
            var isCancel = axios_1.default.isCancel(error);
            var errorMessage;
            var httpStatus;
            var businessCode;
            if (isCancel) {
                errorMessage = error.message || 'Abort';
            }
            else if (error.response) {
                var transform = {
                    httpStatus: error.response.status,
                };
                _this.config.onRespondError(error.response, transform);
                errorMessage = action.failText || transform.message || 'Fail to request api';
                httpStatus = transform.httpStatus;
                businessCode = transform.businessCode;
            }
            else {
                errorMessage = error.message;
                if (/^timeout\sof\s\d+m?s\sexceeded$/i.test(errorMessage)) {
                    errorMessage = _this.config.timeoutMessage ? _this.config.timeoutMessage(errorMessage) : errorMessage;
                }
                else if (/Network\sError/i.test(errorMessage)) {
                    errorMessage = _this.config.networkErrorMessage ? _this.config.networkErrorMessage(errorMessage) : errorMessage;
                }
            }
            var errorResponse = tslib_1.__assign(tslib_1.__assign({}, action), { response: error.response || {}, type: fail, message: errorMessage, httpStatus: httpStatus,
                businessCode: businessCode, effect: action.onFail });
            _this.next(errorResponse);
            if (!isCancel) {
                _this.triggerShowError(errorResponse, action.hideError);
            }
            return Promise.reject(errorResponse);
        });
        // @ts-ignore
        var wrapPromise = promise;
        wrapPromise.cancel = source.cancel;
        return wrapPromise;
    };
    return HttpService;
}(BaseHttpService_1.BaseHttpService));
exports.HttpService = HttpService;
